/*
** Module   : MenuItem.java
** Abstract : MenuItem abstract client implementation
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------------Description-----------------------------
** 001 VIG 20150116 Created initial version.
** 002 HC  20150323 Removed business logic from config classes.
** 003 VIG 20150401 Added doLayout method and is* sub-type methods.
*/
package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * <code>MenuItem</code> client abstract implementation class. 
 */
public abstract class MenuItem<O extends OutputManager<?>>
extends AbstractWidget<O>
implements WidgetWithConfig<MenuItemConfig>
{
   /** Frame attribute container. */
   protected MenuItemConfig config = null; 

   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public MenuItem(WidgetId id, MenuItemConfig cfg)
   {
      
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, MenuItemConfig cfg)
   {
      config = ConfigManager.getInstance().getActiveConfig(id);
      if (config == null)
      {
         config = new MenuItemConfig(id);
      }
      ConfigManager.getInstance().replaceConfig(config, cfg);
   }   

   /**
    * Provide access to the attributes stored in the widget.
    * 
    * @return  Reference to attribute container.
    */
   @SuppressWarnings("unchecked")
   @Override
   public MenuItemConfig config()
   {
      return config;
   }

   /**
    * Make some layout settings for this component such as location and dimension.
    */
   public void doLayout()
   {
      
   }
   
   /**
    * Check if widget is focus traversable. SKIP and RULE aren't traversable.
    * 
    * @return  <code>true</code> if widget is focus traversable.
    */
   @Override
   public boolean focusTraversable()
   {
      return isNormal() ? super.focusTraversable() : false;
   }

   /**
    * The method is called after the configuration associated with 
    * the implementor changes. The parameter points to a reference
    * representing the original configuration state. This reference
    * can be used to detect what configuration fields changed and so
    * optimize any processing related to the configuration change.
    * 
    * @param    beforeUpdate
    */
   @Override
   public void afterConfigUpdate(MenuItemConfig beforeUpdate)
   {
      super.afterConfigUpdateBase(beforeUpdate);
   }
   /**
    * Check if widget needs in cursor to be on when having the input focus.  By default this
    * method always returns <code>true</code> so it needs to be overwritten if other return value
    * is required for particular widget implementation class.
    * 
    * @return  <code>true</code> as the default value.
    */
   @Override
   public boolean forceCursorOnFocus()
   {
      return false;
   }

   /**
    * Check sub-type attribute if it is NORMAL or not (RULE or SKIP)
    * 
    * @return   <code>true</code> if NORMAL <code>false</code> otherwise.
    */
   public boolean isNormal()
   {
      return MenuItemConfig.SBTP_NORMAL.equals(config.subtype);
   }

   /**
    * Check sub-type attribute if it is RULE or not.
    * 
    * @return   <code>true</code> if RULE <code>false</code> otherwise.
    */
   public boolean isRule()
   {
      return MenuItemConfig.SBTP_RULE.equals(config.subtype);
   }
   
   /**
    * Check sub-type attribute if it is SKIP or not.
    * 
    * @return   <code>true</code> if SKIP <code>false</code> otherwise.
    */
   public boolean isSkip()
   {
      return MenuItemConfig.SBTP_SKIP.equals(config.subtype);
   }
 
   /**
    * Check if menu-item was defined in menu definition class as disabled initially.
    * 
    * @return   <code>true</code> if menu-item was defined as disabled.
    */
   public boolean isInitDisabled()
   {
      return config().initDisabled != null && config().initDisabled.booleanValue(); 
   }

   /**
    * Process key event.
    * 
    * @param   event
    *          Key event to process.
    */
   @Override
   public void processKeyEvent(KeyInput event)
   {
      if (event.isConsumed())
      {
         return;
      }

      int key = event.keyCode();
      int actionCode = event.actionCode();
      
      // pop-up menu has simple navigation: DOWN, RIGHT - next focus; LEFT, UP - previous focus.
      Menu<?> menu = Menu.getMenu(this);
      if (menu.config().popupOnly)
      {
         if (actionCode == Keyboard.KA_CURSOR_DOWN || actionCode == Keyboard.KA_CURSOR_RIGHT)
         {
            ThinClient.nextFocus(this);
            parent().repaint();
            return;
         }
         else if (actionCode == Keyboard.KA_CURSOR_UP || actionCode == Keyboard.KA_CURSOR_LEFT)
         {
            ThinClient.previousFocus(this);
            parent().repaint();
            return;
         }
      }
      
      // menu-item choosing
      if (key == Key.VK_ENTER ||
         (Menu.isMenubarElement(this) && actionCode == Keyboard.KA_CURSOR_DOWN) ||       
         (Menu.isMenubarElement(this) && actionCode == Keyboard.KA_CURSOR_UP))
      {
         KeyInput itmEvt;
         MenuItemConfig cfg = config();
         if (cfg.toggleBox)
         {
            cfg.checked = !cfg.checked;
            ConfigHelper.setModified(cfg, true);
            itmEvt = new KeyInput(this, EventType.KEY_PRESSED, Keyboard.SE_VALUE_CHANGED, true);
            EventManager.postEvent(itmEvt);
         }
         else
         {
            //itmEvt = new KeyInput(this, EventType.KEY_PRESSED, Keyboard.SE_CHOOSE, true);
            EventManager.postEvent(new ItemEvent(this));
         }

         return;
      }

      // next focus from menu-item
      if ((!Menu.isMenubarElement(this) && actionCode == Keyboard.KA_CURSOR_DOWN) || 
          (Menu.isMenubarElement(this) && actionCode == Keyboard.KA_CURSOR_RIGHT))
      {
         ThinClient.nextFocus(this);
         parent().repaint();
         return;
      }

      // previous focus from menu-item
      if ((!Menu.isMenubarElement(this) && actionCode == Keyboard.KA_CURSOR_UP) || 
          (Menu.isMenubarElement(this) && actionCode == Keyboard.KA_CURSOR_LEFT))
      {
         ThinClient.previousFocus(this);
         parent().repaint();
         return;
      }

      // hiding sub-menu from menu-item: only current if LEFT or all if RIGHT 
      if ((!Menu.isMenubarElement(this) && actionCode == Keyboard.KA_CURSOR_RIGHT) || 
          (!Menu.isMenubarElement(this) && actionCode == Keyboard.KA_CURSOR_LEFT))
      {
         boolean recursive = actionCode == Keyboard.KA_CURSOR_RIGHT;
         if (parent() instanceof SubMenu)
         {
            ((SubMenu<?>) parent()).hideBody(recursive);

            if (actionCode == Keyboard.KA_CURSOR_RIGHT)
            {
               SubMenu<?> topSubmenu = (SubMenu<?>) parent();
               while (topSubmenu.parent() instanceof SubMenu<?>)
               {
                  topSubmenu = (SubMenu<?>) topSubmenu.parent();
               }

               if (Menu.isMenubarElement(topSubmenu))
               {
                  ThinClient.nextFocus(topSubmenu);
               }
            }
            else if (Menu.isMenubarElement((SubMenu<?>) parent()))
            {
               ThinClient.previousFocus(parent());
            }
         } 
         
         return;
      }
         
      // mnemonic
      if (event.keyChar() != KeyInput.CHAR_UNDEFINED || 
          event.isRealKey() ||
          actionCode == Keyboard.SE_VALUE_CHANGED ||
          actionCode == Keyboard.SE_CHOOSE)
      {
         parent().processKeyEvent(event);
      }
      else
      {
         super.processKeyEvent(event);
      }
      
   }
}
