/*
** Module   : MenuChuiImpl.java
** Abstract : CHUI client implementation of Menu.
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------------Description-----------------------------
** 001 VIG 20150116 Created initial version.
** 002 HC  20150323 Removed business logic from config classes.
** 003 VIG 20150226 Added processKeyEvent method.
*/
package com.goldencode.p2j.ui.client.chui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.border.*;
import com.goldencode.p2j.ui.client.chui.driver.ChuiOutputManager;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * <code>Menu</code> CHUI client implementation class. 
 */
@SuppressWarnings("unchecked")
public class MenuChuiImpl
extends Menu<ChuiOutputManager>
{
   /** Border around the panel. */
   private Border<ChuiOutputManager> border = null;
   
   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public MenuChuiImpl(WidgetId id, MenuConfig cfg)
   {
      super(id, cfg);
      
      if (cfg.popupOnly)
      {
         border = new LineBorder();
      }
   }
   
   /**
    * Drawing worker method.
    */
   public void drawPanel()
   {
      screen().clear(WidgetId._DEFAULT_WINDOW_ID, 
               location(), 
               dimension(false), 
               screen().getInteractiveOutput().widgetColor(this));

      if (border != null)
         border.drawBorder(this, dimension(false));
   }

   /**
    * Draw the peer container. The methods calls the draw() method for
    * ScrollPane instance for the peer container. In addition the required
    * clipping rectangle is calculated for ChildFrame containers because they
    * must be rendered inside the parent container.
    * <p>
    * Drawing for the redirected terminal differs. Labels/headers are drawn
    * only once per page, instead of the usual once per menu.
    */
   @Override
   public void draw()
   {
      drawPanel();
      super.draw();
   }

   /**
    * Get children location for one child.
    * 
    * @param   child
    *          Child component.
    *          
    * @return  Point - parents location
    */
   @Override
   public Point childrenLocation(Widget<ChuiOutputManager> child)
   {
      Point bodyLoc = location();
      double x = config().popupOnly ? bodyLoc.x + 1 : bodyLoc.x; 
      return new Point(x, bodyLoc.y); // border
   }
   
   /**
    * Hide/show widget.
    * <p>
    * If the menu is being shown and not yet attached to a window, attached it to the default
    * window.
    * 
    * @param   visible
    *          New visible state.
    */
   @Override
   public void setVisible(boolean visible)
   {
      setDisplayed(visible);
      super.setVisible(visible);
      
      if (visible && parent() == null)
      {
         setParent(Window.<ChuiOutputManager>getDefaultWindow());
      }
      
      for(Widget<ChuiOutputManager> w : widgets())
      {
         w.setVisible(visible);
      }
   }

   /**
    * Replace original version to take into account P2J specific behavior.
    * 
    * @see  com.goldencode.p2j.ui.client.widget.Widget#dimension()
    */
   @Override
   public Dimension dimension()
   {
      return dimension(true);
   }
   
   /**
    * Lay out container. Set children locations.
    */
   @Override
   public void doLayout()
   {
      setLocation(calcLocation());
      super.doLayout();
      
      Widget<ChuiOutputManager>[] w = widgets();
      double x=0;
      
      for(int i=0; i<w.length; i++)
      {
         if (!config().popupOnly) // menu-bar
         {
            w[i].setLocation(new Point(x, 0));
            x += 1 + w[i].width(); // shift horizontally and add space
         }
         else
         {
            w[i].setLocation(new Point(1, i + 1)); // borders
         }
         
         if(w[i] instanceof SubMenu)
         {
            ((SubMenu<?>)w[i]).doLayout();
         }
      }
   }
   
   /**
    * Calculates MENU location relative to its parent if it is POPUP-ONLY or 
    * [0, 0] if it is MENUBAR.
    * 
    * @return   point of location relative to its parent.
    */
   private Point calcLocation()
   {
      if (config().popupOnly)
      {
         Widget<?> owner = screen().getRegistry().getComponent(config().ownerId);
         if (owner != null)
         {
            AbstractWidget<?> awOwner = (AbstractWidget<?>) owner;
            Point ownloc = awOwner.location();
            
            // pop-up menu is shown right under owner, shifted by 1 horizontally 
            return new Point(ownloc.x + 1, ownloc.y + awOwner.height() + 1);
         }
      }
      
      return new Point(0, 0); // menu-bar is always on top
   }
   
   /**
    * Replace original version to take into account P2J specific behavior.
    * 
    * @see  com.goldencode.p2j.ui.client.widget.Widget#dimension()
    * 
    * @param   includeNested
    *          <code>true</code> if we should process nested sub-menus
    */
   private Dimension dimension(boolean includeNested)
   {
      double height = SubMenuChuiImpl.getMaxItemHeight(this, includeNested);
      double width = SubMenuChuiImpl.getMaxItemWidth(this, includeNested);
      
      if (!config().popupOnly)
      {
         double menubarWidth = 0; 
         for(Widget<?> w : widgets())
         {
            AbstractWidget<?> aw = (AbstractWidget<?>) w;
            menubarWidth += aw.width() + 1; // space between items in a menu-bar
         }
         width = Math.max(menubarWidth, width);
      }
      
      return new Dimension(width + 4, height); // left and right borders with insets
   }
}
